@@ -1,5 +1,5 @@
 /* Internal definitions for libdwarf.
-   Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007 Red Hat, Inc.
+   Copyright (C) 2002-2009 Red Hat, Inc.
    This file is part of Red Hat elfutils.
    Written by Ulrich Drepper <drepper@redhat.com>, 2002.
 
@@ -62,7 +62,7 @@
 
 
 /* Version of the DWARF specification we support.  */
-#define DWARF_VERSION 2
+#define DWARF_VERSION 3
 
 /* Version of the CIE format.  */
 #define CIE_VERSION 1
@@ -194,12 +194,12 @@ struct Dwarf
 /* Abbreviation representation.  */
 struct Dwarf_Abbrev
 {
+  Dwarf_Off offset;
+  unsigned char *attrp;
+  unsigned int attrcnt;
   unsigned int code;
   unsigned int tag;
-  int has_children;
-  unsigned int attrcnt;
-  unsigned char *attrp;
-  Dwarf_Off offset;
+  bool has_children;
 };
 
 #include "dwarf_abbrev_hash.h"
@@ -208,7 +208,7 @@ struct Dwarf_Abbrev
 /* Files in line information records.  */
 struct Dwarf_Files_s
   {
-    Dwarf *dbg;
+    struct Dwarf_CU *cu;
     unsigned int ndirs;
     unsigned int nfiles;
     struct Dwarf_Fileinfo_s
@@ -223,26 +223,27 @@ typedef struct Dwarf_Fileinfo_s Dwarf_Fileinfo;
 
 
 /* Representation of a row in the line table.  */
+
+struct Dwarf_Line_s
+{
+  Dwarf_Addr addr;
+  unsigned int file;
+  int line;
+  unsigned short int column;
+  unsigned int is_stmt:1;
+  unsigned int basic_block:1;
+  unsigned int end_sequence:1;
+  unsigned int prologue_end:1;
+  unsigned int epilogue_begin:1;
+
+  Dwarf_Files *files;
+};
+
 struct Dwarf_Lines_s
-  {
-    size_t nlines;
-
-    struct Dwarf_Line_s
-    {
-      Dwarf_Addr addr;
-      unsigned int file;
-      int line;
-      unsigned short int column;
-      unsigned int is_stmt:1;
-      unsigned int basic_block:1;
-      unsigned int end_sequence:1;
-      unsigned int prologue_end:1;
-      unsigned int epilogue_begin:1;
-
-      Dwarf_Files *files;
-    } info[0];
-  };
-
+{
+  size_t nlines;
+  struct Dwarf_Line_s info[0];
+};
 
 /* Representation of address ranges.  */
 struct Dwarf_Aranges_s
@@ -286,6 +287,19 @@ struct Dwarf_CU
   void *locs;
 };
 
+/* Compute the offset of a CU's first DIE from its offset.  This
+   is either:
+        LEN       VER     OFFSET    ADDR
+      4-bytes + 2-bytes + 4-bytes + 1-byte  for 32-bit dwarf
+     12-bytes + 2-bytes + 8-bytes + 1-byte  for 64-bit dwarf
+
+   Note the trick in the computation.  If the offset_size is 4
+   the '- 4' term changes the '3 *' into a '2 *'.  If the
+   offset_size is 8 it accounts for the 4-byte escape value
+   used at the start of the length.  */
+#define DIE_OFFSET_FROM_CU_OFFSET(cu_offset, offset_size) \
+  ((cu_offset) + 3 * (offset_size) - 4 + 3)
+
 #define CUDIE(fromcu) \
   ((Dwarf_Die)								      \
    {									      \
